#!/usr/bin/env node
var us = require('underscore');
var fs = require('fs');

var option = {encoding: "utf-8"};


if (process.argv.length < 3)
{
	console.log("使い方: TK2JISX0213 ToukiShrinkMap.json [Output.json]");
	console.error("引数が足りません。入力ファイルを指定してください。");
	process.exit(-1);
}
else
{
	var 入力ファイル名 = process.argv[2];
	var 出力ファイル名 = "変換テーブル.json";

	if (process.argv.length == 4)
		var 出力ファイル名 = process.argv[3];
	
	// 入力ファイルを読み込む
	json = fs.readFileSync(入力ファイル名, option);
	JISX0213toUCS = fs.readFileSync("JISX0213_UCS.json", option);
	
	var JISX0213_UCS = JSON.parse(JISX0213toUCS);
	var 登記統一文字縮退マップ = JSON.parse(json);

	/*
	 * mapメソッドを利用して、登記統一文字縮退マップの候補情報に、
	 * JIS水準情報と漢字施策情報を付加する。
	 * この処理は、時間を要する。
	 */
	us.map(登記統一文字縮退マップ.content, function(縮退候補情報) {
		us.map(縮退候補情報, function(情報) {
			us.map(情報, function(候補) {
				if (候補.UCS != undefined)
				{
					JIS情報 = us.findWhere(JISX0213_UCS, {"UCS": 候補.UCS});
					if (JIS情報.水準 != undefined)
						候補.水準 = JIS情報.水準;
					
					if (JIS情報.漢字施策 != undefined)
						候補.漢字施策 = JIS情報.漢字施策;
				}
				return 候補;
			});
			return 情報;
		});
		return 縮退候補情報;
	});
	
	一意な変換(登記統一文字縮退マップ);
}

function 一意な変換(登記統一文字縮退マップ)
{
	var 変換テーブル = [];	//一意な変換候補を格納する配列
	
	us.each(登記統一文字縮退マップ.content, function(縮退候補情報) {
		/* 0. 非漢字
		 * 登記統一文字には、非漢字が含まれているので、JIS X 0213に対応または関連する文字については、最初に処理する。
		 *
		 */
		if (縮退候補情報['非漢字'] != undefined)
		{
			変換テーブル.push({
				"登記統一文字番号": 縮退候補情報.登記統一文字番号, 
				"変換先": 変換先候補(縮退候補情報['非漢字'][0]), 
				"備考": "非漢字"
			});
			return ;
		}

		if (縮退候補情報['JIS包摂・UCS統合'] != undefined)
		{
			変換テーブル.push({
				"登記統一文字番号": 縮退候補情報.登記統一文字番号, 
				"変換先": 変換先候補(縮退候補情報['JIS包摂・UCS統合'][0]), 
				"備考": "JIS包摂・UCS統合"
			});
			return ;
		}
		
		/*
		 * 2. 法務省戸籍法関連通達・通知
		 *  法務省の戸籍法関連通達・通知の情報が記載されているもの。
		 */
		if (縮退候補情報['法務省戸籍法関連通達・通知'] != undefined)
		{
			/*
			 * 2.1 別字とされるものの除外
			 *   民一2842号通達 誤俗表で、別字とされているものは、全体から除く。
			 *   別字となっているUCSを特定して、全体に対してフィルタをかける。
			 */
			結果 = us.where(縮退候補情報['法務省戸籍法関連通達・通知'], {"付記":"別字"});	//別字とされているUCSを特定する。
			if (結果.length > 0)
			{
				var UCS = us.pluck(結果, 'UCS');
				
				// rejectメソッドを利用して、別字とされたUCSの値を消す。
				縮退候補情報['法務省戸籍法関連通達・通知'] = us.reject(縮退候補情報['法務省戸籍法関連通達・通知'], 
					function(縮退候補){ return us.contains(UCS, 縮退候補['UCS']) == true; });
				縮退候補情報['法務省告示582号別表第四'] = us.reject(縮退候補情報['法務省告示582号別表第四'], 
					function(縮退候補){ return us.contains(UCS, 縮退候補['UCS']) == true; });
				縮退候補情報['辞書類等による関連字'] = us.reject(縮退候補情報['辞書類等による関連字'], 
					function(縮退候補){ return us.contains(UCS, 縮退候補['UCS']) == true; });
			}

			/*
			 * 2.2 法務省戸籍法関聯通達・通知に関するものが一つの場合、それを選択 */
			if (縮退候補情報['法務省戸籍法関連通達・通知'].length == 1)
			{
				変換テーブル.push({
					"登記統一文字番号": 縮退候補情報.登記統一文字番号, 
					"変換先": 変換先候補(縮退候補情報['法務省戸籍法関連通達・通知'][0]), 
					"備考": 縮退候補情報['法務省戸籍法関連通達・通知'][0]['種別']
				});
				return ;
			}
			else if (縮退候補情報['法務省戸籍法関連通達・通知'].length > 1)
			{
				/* 法務省戸籍法関聯通達・通知に関するものが二つ以上の場合で、正俗表のものがあれば、それを選択 */
				if ((f = us.where(縮退候補情報['法務省戸籍法関連通達・通知'], {"種別":"民二5202号通知別表 正字・俗字等対照表"})).length == 1)
				{
					変換テーブル.push({
						"登記統一文字番号": 縮退候補情報.登記統一文字番号, 
						"変換先": 変換先候補(f[0]), 
						"備考": f[0].種別
					});
					return ;
				}
				else
				{
					/* 戸籍統一文字情報 親字・正字がないもの */
					var 変換先;
					誤俗表 = us.where(縮退候補情報['法務省戸籍法関連通達・通知'], {"種別":"民一2842号通達別表 誤字俗字・正字一覧表"});
					常用漢字 = us.where(誤俗表, {"漢字施策":"常用漢字"});
					人名用漢字 = us.where(誤俗表, {"漢字施策":"人名用漢字"});
					
					
					if (常用漢字.length == 1)
						変換先 = 常用漢字[0];
					else if (常用漢字.length == 0 && 人名用漢字.length == 1)
						変換先 = 人名用漢字[0];
					else if (常用漢字.length == 0 && 人名用漢字.length > 1)
						// 両方とも人名用漢字の場合は、水準の値が小さいものを変換先とする。
						変換先 = us.min(人名用漢字, function(候補){ return 候補.水準;});
					
					変換テーブル.push({
						"登記統一文字番号": 縮退候補情報.登記統一文字番号, 
						"変換先": 変換先候補(変換先),
						"備考": "民一2842号通達別表 誤字俗字・正字一覧表"
					});
					
					return;
				}
			}
		}
		
		/* 
		 * 3. 法務省告示582号別表第四
		 *   別表第四の一により、入管正字への置き換えが示されているものであって、第1順位、第2順位の値がJIS X 0213に含まれるもの
		 *   がある場合。第1順位と第2順位が両方示されている場合は、第1順位を選択する。
		 */
		if (縮退候補情報['法務省告示582号別表第四'] != undefined)
		{
			別表四の一 = us.where(縮退候補情報['法務省告示582号別表第四'], {"表": "一"});
			if ( 別表四の一.length > 0)
			{
				変換テーブル.push({
					"登記統一文字番号": 縮退候補情報.登記統一文字番号, 
					"変換先": 変換先候補(us.first(別表四の一)), 
					"備考": "法務省告示582号別表第四"
				});
				return ;
			}
		}

		/* 候補が無いものは、「＿」(U+FF3F FULLWIDTH LOW LINE)を変換先にする。 */
		変換テーブル.push({
			"登記統一文字番号": 縮退候補情報.登記統一文字番号, 
			"変換先":{"JIS X 0213":"1-01-18", "UCS": "U+FF3F"}, 
			"備考": "候補なし"
		});

	});

	var writeData =  
	{
		"meta": {
			"dct:title": "登記統一文字縮退マップ 一意な変換表",
			"dct:issued": (new Date).toISOString().substr(0, 10),
			"dct:creator": [{
				"foaf:name": {
					"ja": "独立行政法人情報処理推進機構",
					"en": "Information-technology Promotion Agency, Japan"
				},
				"foaf:homepage": "http://www.ipa.go.jp/"
			} //,{ }
			],
			"owl:versionInfo": "",
			"cc:license": "http://creativecommons.org/licenses/by-sa/2.1/jp/"
		},
		"content": 変換テーブル
	}
	
	// JSON形式で出力
	json = JSON.stringify(writeData, null, "\t");
	fs.writeFileSync(出力ファイル名, json);
	
	// TSV形式で出力
	tsv = [];
	us.each(変換テーブル, function(list) {
		tsv.push(us.reduce(us.values(list), function(previous, data) {
			data = (typeof data == 'object') ? us.values(data).join("\t") : data;
			
			if (previous == "")
				return data;
			else
				return previous + "\t" + data;
		}, ""));
	});
	fs.writeFileSync(出力ファイル名.replace(/\..*/, ".tsv"), tsv.join("\r\n"));
}

function 変換先候補(情報)
{
	return {"JIS X 0213": 情報["JIS X 0213"], "UCS": 情報["UCS"]};
}
